/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "graphics.h"
#include "models.h"
#include "gui.h"

#include "graphics.inl"
#include "gui.inl"


namespace magnum { namespace sample {
using namespace magnum::graphics;
using namespace magnum::graphics3d;
using namespace magnum::scene3d;
using magnum::graphics3d::Context;


static Camera*			g_Camera;
static Mode*			g_Mode;
static Model*			g_Model;
static PointLight*		g_Light;
static ParameterBuffer* g_Parameters;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onInit() {
	g_Camera = new Camera();
	g_Camera->setNearPlane(10);
	g_Camera->setFarPlane(800);
	g_Camera->select();

	g_Mode = new Mode();
	g_Mode->setDepthTest(Mode::Less);
	g_Mode->setDepthWrite(true);

	g_Light = new PointLight();
	g_Light->setPosition(Vector3<float>(900.0f,790.0f,400.0f));
	g_Light->setAmbient(Colour<float>(0.0f,0.0f,0.0f));
	g_Light->setDiffuse(Colour<float>(1.0f,1.0f,1.0f));
	g_Light->setSpecular(Colour<float>(0.3f,0.3f,0.3f));

	MeshList mlist;
	g_Model = create3ds();
	((StaticModel*)g_Model)->getMeshes(&mlist);
	setTextureMaterial(&mlist);
//	setTexEnvBumpMaterial(g_Model);
	g_Model->optimize();

	g_Parameters = ParameterBuffer::create(g_Model->getParameters());

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onExit() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void GraphicsView::onRender() {
	g_Mode->select();

	m_Frame->clear(FrameBuffer::ClearColour | FrameBuffer::ClearDepth | FrameBuffer::ClearStencil);

	m_MatrixStack->selectMode(MatrixStack::ModelView);
	m_MatrixStack->identity();

	g_Light->select(0);
	m_Context->getLightStack()->enableLighting(LightStack::TwoSided | LightStack::LocalViewer);

	m_Context->setCulling(Context::CullBack);
	*(Matrix4<float>*)g_Parameters->getBuffer() = (Matrix4<float>)getTransform();
	g_Model->render(g_Parameters);
}

}; };

