/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "magnum.h"
#include "models.h"

namespace magnum { namespace sample {
using namespace magnum::graphics;
using namespace magnum::graphics3d;
using namespace magnum::io;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void setShader(MeshList* meshes, Shader* shader) {
	MeshList::Iterator iter;
	for (iter = meshes->begin(); iter != meshes->end(); iter++) {
		Appearance* app = (*iter)->getAppearance();
		if (app == NULL)
			continue;
		app->attachShader(shader);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void setTexture(MeshList* meshes, Texture* tex, unsigned int stage) {
	MeshList::Iterator iter;
	for (iter = meshes->begin(); iter != meshes->end(); iter++) {
		Appearance* app = (*iter)->getAppearance();
		if (app == NULL)
			continue;
		app->attachTexture(tex, stage);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void setTextureMaterial(MeshList* meshes) {
	Shader* shader = TextureShader::create();
	setShader(meshes, shader);
	shader->release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void setBumpMaterial(MeshList* meshes) {
	Shader* shader = BumpShader::create();
	setShader(meshes, shader);
	shader->release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void setEnvMaterial(MeshList* meshes) {
	TranslatorJPG j;
	Shader* shader = EnvironmentShader::create();
	File* file = SystemFile::open("data\\env.jpg");
	Image* img = j.decode(file->getInputStream());
	Texture* env = Texture2D::create(img, Texture::GenerateAll);
	env->setFilter(Texture::LinearMipmapLinear, Texture::Linear);
	file->release();
	img->release();

	setShader(meshes, shader);
	setTexture(meshes, env, 0);

	shader->release();
	env->release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void setEnvBumpMaterial(MeshList* meshes) {
	TranslatorJPG j;
	Shader* shader = EnvironmentBumpShader::create();
	File* file = SystemFile::open("data\\env.jpg");
	Image* img = j.decode(file->getInputStream());
	Texture* env = Texture2D::create(img, Texture::GenerateAll);
	env->setFilter(Texture::LinearMipmapLinear, Texture::Linear);
	file->release();
	img->release();

	setShader(meshes, shader);
	setTexture(meshes, env, 1);

	shader->release();
	env->release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void setTexEnvBumpMaterial(MeshList* meshes) {
	TranslatorJPG j;
	Shader* shader = TextureEnvironmentBumpShader::create();
	Texture* tex;
	File* file = SystemFile::open("data\\env.jpg");
	Image* img = j.decode(file->getInputStream());
	Texture* env = Texture2D::create(img, Texture::GenerateAll);
	env->setFilter(Texture::LinearMipmapLinear, Texture::Linear);
	file->release();
	img->release();

	MeshList::Iterator iter;
	for (iter = meshes->begin(); iter != meshes->end(); iter++) {
		Appearance* app = (*iter)->getAppearance();
		if (app == NULL)
			continue;
		app->attachShader(shader);
		tex = app->getTexture(0);
		if (tex != NULL) {
			app->attachTexture(tex,1);
			app->attachTexture(env,2);
		}
	}

	shader->release();
	env->release();
}

}; };
